import socket
import sys
import struct
import threading
import fcntl, os
import errno

sys.path.append("../ExternalCall")


# For this example you also need to copy the example "ExternalCall" to the same directory as this example.
from uds_external_function import ExternalFunctionBase

# The name of the class defines the name of the IO driver instance.
# That is "UDS_IODriver_", followed by the instance index in ascending order (starting with 0).
class UDS_IODriver_0(ExternalFunctionBase):

    # This is the effective call of IO update. The inputs of this function are the outputs of the IEC application and vice versa.
    # Input parameters (IEC outputs) are passed as dict of tuple of strings (paramType, paramValue) with parameter ID from devdesc.xml as index.
    # The driver can handle the following return datatypes: USINT, UINT, UDINT, ULINT. Other datatypes should be casted to one of those types.
    # E.g. {'2000': ('UDINT', '11'), '2001': ('UDINT', '22')}
    # Return parameters must be in the same format. E.g. {'1000': ('UDINT', '33'), '1001': ('UDINT', '44')}
    # In this example, the output values of the IO device are just mapped to the inputs.
    def Call(self, dictParams):
        dictRetParams = {}
        if dictParams != {}:
            out1 = 0
            out2 = 0
            if '2000' in dictParams:
                out1 = int(dictParams['2000'][1])
            if '2001' in dictParams:
                out2 = int(dictParams['2001'][1])

            dictRetParams['1000'] = 'UDINT', out1
            dictRetParams['1001'] = 'UDINT', out2

        return dictRetParams   

# example for external function:
if __name__ == "__main__":

    extFunc = UDS_IODriver_0()
    extFunc.start()


