import socket
import sys, os
import struct
import threading
import fcntl
import termios
import time
import inspect

#
# this example shows how to use the plcshell of the CODESYS control runtime
# the mechanism uses unix domain sockets
# 
#


# This class helps to connect to a CODESYS Control runtime
# and listen to the internal event mechanism of CODESYS runtimes
class CODESYS_PlcShell():

    # server endpoint for events:
    endpoint = '/var/run/codesysextension/plcshell.sock'

    def __init__(self):
        pass
    
    # returns tuple of (result, response)
    # result = True | False
    # response = <string>
    def _executecmd(self, command, rxsize=1024, bVerbose = False, delay = 1):
        sock = None
        command = command + "\n" # dont forget to press Enter after the command
        response = ""
        result = False
        commandbytes = str.encode(command)

        try:
            sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
            if bVerbose:
                print("connecting to runtime plcshell: " + CODESYS_PlcShell.endpoint)

            sock.connect(CODESYS_PlcShell.endpoint)

            if bVerbose:
                print("send:" + command)
            sock.sendall(commandbytes)
            time.sleep(delay)
            if bVerbose:
                print("waiting for reply...")
            responsebytes = sock.recv(rxsize)
            if len(responsebytes) > 0:
                response = bytes.decode(responsebytes)
                if bVerbose:
                    print("reply: " + response)
            
                
        except socket.error as msg:
            print("error: " + str(msg))
            result = False
        finally:
            if sock != None:
                sock.close()

        return result, response

    def rtsinfo(self):
        result, r = self._executecmd("rtsinfo")
        return r

    def rtcget(self):
        result, r = self._executecmd("rtc-get")
        return r
    
    def plcload(self):
        result, r = self._executecmd("plcload")
        return r

    def applist(self):
        result, r = self._executecmd("applist")
        return r

    def startprg(self, applicationname):
        result, r = self._executecmd("startprg " + applicationname)
        return r
        
    def stopprg(self, applicationname):
        result, r = self._executecmd("stopprg " + applicationname)
        return r
    
    def startallprg(self):
        result, r = self._executecmd("startprg all")
        return r
        
    def stopallprg(self):
        result, r = self._executecmd("stopprg all")
        return r
        

# end of class


# example for event handlers:
if __name__ == "__main__":

    plcshell = CODESYS_PlcShell()
    
    r = plcshell.rtsinfo()
    print(r)

    r = plcshell.rtcget()
    print(r)
    
    r = plcshell.plcload()
    print(r)
        
    r = plcshell.applist()
    print(r)

    r = plcshell.startprg("Application")
    print(r)
    
    r = plcshell.stopprg("Application")
    print(r)

    r = plcshell.startallprg()
    print(r)
    
    r = plcshell.stopallprg()
    print(r)

